import { lazy, LazyExoticComponent, ReactNode, Suspense, useEffect } from "react";
import { ErrorBoundary } from "react-error-boundary";
import { Route, Routes, useSearchParams } from "react-router-dom";
import { CenteredLoadingSpinner } from "./components/common/Loading";
import { CenteredErrorFallback } from "./components/common/Fallback";
import { Links, rootLink } from "./util/links";

// Lazy Views
const ProjectView = lazy(() => import("@src/views/ProjectView"));

// Static Views
import AutoRedirect from "./views/AutoRedirect";
import AuthenticateView from "./views/AuthenticateView";
import { useAuth } from "./ctx/AuthContext";
import AuthorizedView from "./views/AuthorizedView";

declare type Portal = { path: Links; view: ReactNode };

function Auth(props: { view: (() => ReactNode) | LazyExoticComponent<() => ReactNode> }) {
  const { auth, loading, initialized } = useAuth();
  const Component = props.view;
  if (!loading && !initialized && !!auth) return <Component />;
  return <AutoRedirect />;
}

const lazyPortals: Portal[] = [{ path: Links.ProjectView, view: <Auth view={ProjectView} /> }];

// Raw Routes
const rawPortals: Portal[] = [
  { path: Links.Authenticate, view: <AuthenticateView /> },
  { path: Links.Authorized, view: <Auth view={AuthorizedView} /> },
];

export default function Portal() {
  return (
    <Routes>
      {lazyPortals.map((p: Portal, i) => (
        <Route
          key={i}
          path={rootLink(p.path)}
          element={
            <Suspense fallback={<CenteredLoadingSpinner />}>
              <ErrorBoundary fallback={<CenteredErrorFallback />} key={p.path}>
                <Suspense fallback={<CenteredLoadingSpinner />} key={p.path}>
                  {p.view}
                </Suspense>
              </ErrorBoundary>
            </Suspense>
          }
        />
      ))}

      {rawPortals.map((p: Portal, i) => (
        <Route key={i} path={rootLink(p.path)} element={p.view} />
      ))}
      <Route path={"*"} element={<AutoRedirect />} />
    </Routes>
  );
}
