import { useState } from "react";
import { Link } from "react-router-dom";
import { Box, Flex, IconButton, Text, List, ListItem, ListIcon, Tooltip, Divider, Icon, Slide } from "@chakra-ui/react";
import { Links } from "../util/links";
import { GuardedContent } from "../util/guards";
import { MdSettings, MdMenu, MdLogout, MdLock } from "react-icons/md";
import AppGuard from "@lib/Guards";
import { useAuth } from "@src/ctx/AuthContext";
import { Policy } from "@lib/Policies";

const layouts = [
  {
    name: "Settings",
    link: Links.Settings,
    icon: MdSettings,
    guards: [...AppGuard.IAMAuthenticated],
  },
];

export const drawerWidth = "230px";
export const miniDrawerWidth = "5rem";

export function useMainMenu(defaultState: boolean = false): [boolean, () => void, string] {
  const [minified, setMinified] = useState<boolean>(defaultState);
  const toggleMinified = () => setMinified(!minified);
  const width = minified ? miniDrawerWidth : drawerWidth;
  return [minified, toggleMinified, width];
}

export default function MainMenu(props: { minified: boolean; toggleMinified: () => void; width: string }) {
  const { logout, activePolicies, auth } = useAuth();
  const menus = layouts.filter(({ guards: g }) => g.length === 0 || Policy.multiAuthorizedTo(activePolicies, g));
  const menuButtonSx = { height: "3rem", _hover: { bg: "rgba(0,0,0,.1)" }, w: "100%" };

  const slideMotion = { animate: { width: props.width }, initial: { width: props.width } };

  return (
    <Box position="fixed" zIndex={1}>
      <Slide direction="left" in={props.minified} motionProps={slideMotion}>
        <Box w={props.width} maxW={props.width} bg="background.paper" h="100%">
          <Box p={4} height="100%" display="flex" flexDirection="column">
            <Flex mb={4} h="48px">
              <IconButton
                icon={<Icon as={MdMenu} />}
                onClick={props.toggleMinified}
                aria-label="Toggle menu"
                w="48px"
                h="100%"
                p="0"
              />
              <GuardedContent guard={!props.minified}>
                <Text ml="1rem" lineHeight="48px">
                  Cairo
                </Text>
                <img
                  src={`${import.meta.env.BASE_URL}icons/favicon.ico`}
                  alt=""
                  style={{ height: "1.5rem", marginLeft: ".75rem", marginTop: "auto", marginBottom: "auto" }}
                />
              </GuardedContent>
            </Flex>
            <Divider />
            <List spacing={3} mt={2}>
              {menus.map((menuItem, index) => (
                <ListItem key={index} w="100%">
                  <Tooltip label={menuItem.name} placement="right" isDisabled={props.minified}>
                    <Flex as={Link} to={menuItem.link} align="center" px={4} py={2} borderRadius="md" {...menuButtonSx}>
                      <ListIcon as={menuItem.icon} />
                      <GuardedContent guard={!props.minified}>
                        <Text ml={2}>{menuItem.name}</Text>
                      </GuardedContent>
                    </Flex>
                  </Tooltip>
                </ListItem>
              ))}
              <GuardedContent guard={auth}>
                <ListItem w="100%">
                  <Flex as="button" onClick={logout} align="center" px={4} py={2} borderRadius="md" {...menuButtonSx}>
                    <ListIcon as={MdLogout} />
                    <GuardedContent guard={!props.minified}>
                      <Text ml={2}>Logout</Text>
                    </GuardedContent>
                  </Flex>
                </ListItem>
              </GuardedContent>
              <GuardedContent guard={!auth}>
                <ListItem w="100%">
                  <Flex as={Link} to={Links.Authenticate} align="center" px={4} py={2} borderRadius="md" {...menuButtonSx}>
                    <ListIcon as={MdLock} />
                    <GuardedContent guard={!props.minified}>
                      <Text ml={2}>Login</Text>
                    </GuardedContent>
                  </Flex>
                </ListItem>
              </GuardedContent>
            </List>
          </Box>
        </Box>
      </Slide>
    </Box>
  );
}
