import { ReactNode, createContext, useContext, useMemo, useReducer } from "react";
import { apiRequest } from "@dunemask/vix/bridge";
import { Policy, PolicyString } from "@lib/Policies";
import { Resource } from "@lib/vix/AppResources";
import { CDatabaseContract } from "@lib/contracts/database.contracts";
import { useSearchParams } from "react-router-dom";

const project = import.meta.env.VITE_CAIRO_PROJECT as string;
const credentialApiPath = `auth/credentials`;

export enum AuthStorageKeys {
  USER = "user",
  USER_TOKEN = "user-token",
  SPOOF_USER = "spoof-user",
  SPOOF_TOKEN = "spoof-token",
}

export interface CredentialDTO {
  user: CDatabaseContract["User"];
  policies: PolicyString[];
}

export interface LoginCredentialDTO extends CredentialDTO {
  token: string;
}

type LoginAction = { type: AuthAction.LOGIN; token: string; user: any; policies: Policy[] };
type LogoutAction = { type: AuthAction.LOGOUT };
type SpoofAction = { type: AuthAction.SPOOF; token: string; user: any; policies: Policy[] };
type UnspoofAction = { type: AuthAction.UNSPOOF };
type LoadingAction = { type: AuthAction.LOADING; loading: boolean };
type InitializeAction = { type: AuthAction.INTIALIZE; initialized: boolean };
type Action = LoginAction | LogoutAction | SpoofAction | UnspoofAction | LoadingAction | InitializeAction;

enum AuthAction {
  LOGIN,
  LOGOUT,
  SPOOF,
  UNSPOOF,
  LOADING,
  INTIALIZE,
}

interface AuthState {
  user?: CDatabaseContract["User"];
  auth: boolean;
  token?: string;
  userPolicies: Policy[];
  spoofUser?: CDatabaseContract["User"];
  spoofToken?: string;
  spoofPolicies: Policy[];
  activeUser?: CDatabaseContract["User"];
  activeToken?: string;
  activePolicies: Policy[];
  loading: boolean;
  initialized: false;
}

interface AuthContextType extends AuthState {
  logout: () => void;
  login: (userData: CDatabaseContract["User"], token: string, policies?: Policy[]) => Promise<void>;
  spoof: (userData: CDatabaseContract["User"], token: string, policies?: Policy[]) => Promise<void>;
  unspoof: () => void;
  authInit: () => Promise<void>;
}

const initialState: AuthContextType = {
  auth: false,
  userPolicies: [],
  spoofPolicies: [],
  activePolicies: [],
  logout: () => {},
  login: async () => {},
  spoof: async () => {},
  unspoof: () => {},
  authInit: async () => {},
  loading: false,
  initialized: false,
};

const AuthContext = createContext<AuthContextType>(initialState);

export const useAuth = () => useContext<AuthContextType>(AuthContext);

function onLogout(state: AuthState): AuthState {
  window.localStorage.removeItem(AuthStorageKeys.USER);
  window.localStorage.removeItem(AuthStorageKeys.USER_TOKEN);
  window.localStorage.removeItem(AuthStorageKeys.SPOOF_USER);
  window.localStorage.removeItem(AuthStorageKeys.SPOOF_TOKEN);
  return {
    ...state,
    auth: false,
    token: undefined,
    spoofToken: undefined,
    user: undefined,
    spoofUser: undefined,
    activeUser: undefined,
    activeToken: undefined,
    activePolicies: [],
  };
}

function onLogin(state: AuthState, action: LoginAction): AuthState {
  const { token, user, policies: userPolicies } = action;
  window.localStorage.setItem(AuthStorageKeys.USER, JSON.stringify(user));
  window.localStorage.setItem(AuthStorageKeys.USER_TOKEN, token);
  const [activeToken, activeUser, activePolicies] = [token, user, userPolicies];
  return { ...state, auth: true, token, user, activeToken, activeUser, activePolicies, userPolicies };
}

function onSpoof(state: AuthState, action: SpoofAction): AuthState {
  const { token: spoofToken, user: spoofUser, policies: spoofPolicies } = action;
  window.localStorage.setItem(AuthStorageKeys.SPOOF_USER, JSON.stringify(spoofUser));
  window.localStorage.setItem(AuthStorageKeys.SPOOF_TOKEN, spoofToken);
  const [activeToken, activeUser, activePolicies] = [spoofToken, spoofUser, spoofPolicies];
  return { ...state, auth: true, spoofToken, spoofUser, activeToken, activeUser, activePolicies, spoofPolicies };
}

function onUnspoof(state: AuthState): AuthState {
  const { token, user } = state;
  window.localStorage.removeItem(AuthStorageKeys.SPOOF_USER);
  window.localStorage.removeItem(AuthStorageKeys.SPOOF_TOKEN);
  const [activeToken, activeUser, activePolicies] = [token, user, state.userPolicies];
  return {
    ...state,
    auth: true,
    spoofToken: undefined,
    spoofUser: undefined,
    activeToken,
    activeUser,
    spoofPolicies: [],
    activePolicies,
  };
}

const onLoading = (state: AuthState, action: LoadingAction) => ({ ...state, loading: action.loading }) as AuthState;

const onInitialized = (state: AuthState, action: InitializeAction) =>
  ({ ...state, initialized: action.initialized }) as AuthState;

function authReducer(state: AuthState, action: Action): AuthState {
  if (action.type === AuthAction.LOGIN) return onLogin(state, action);
  if (action.type === AuthAction.SPOOF) return onSpoof(state, action);
  if (action.type === AuthAction.LOGOUT) return onLogout(state);
  if (action.type === AuthAction.UNSPOOF) return onUnspoof(state);
  if (action.type === AuthAction.LOADING) return onLoading(state, action);
  if (action.type === AuthAction.INTIALIZE) return onInitialized(state, action);
  return state;
}

export const AuthProvider = ({ children }: { children: ReactNode }) => {
  const [state, dispatch] = useReducer(authReducer, initialState);
  const [search] = useSearchParams();

  async function getPolicies(token: string): Promise<Policy[]> {
    const extraHeaders = { Authorization: `Bearer ${token}` };
    const projectId = !!search.get("projectId") ? search.get("projectId") : project;
    const apiPath = `/${projectId}/${credentialApiPath}`;
    const credentials = await apiRequest({ subpath: apiPath, jsonify: true, extraHeaders });
    if (!credentials) return [];
    const { policies } = credentials as CredentialDTO;
    return Policy.parseResourcePolicies<Resource>(policies);
  }

  async function login(userData: CDatabaseContract["User"], token: string, policies?: Policy[]) {
    dispatch({ type: AuthAction.LOADING, loading: true });
    const userPolicies = !!policies ? policies : await getPolicies(token);
    dispatch({ type: AuthAction.LOGIN, policies: userPolicies, user: userData, token });
    dispatch({ type: AuthAction.LOADING, loading: false });
  }

  async function spoof(userData: CDatabaseContract["User"], token: string, policies?: Policy[]) {
    dispatch({ type: AuthAction.LOADING, loading: true });
    const userPolicies = !!policies ? policies : await getPolicies(token);
    dispatch({ type: AuthAction.SPOOF, policies: userPolicies, user: userData, token });
    dispatch({ type: AuthAction.LOADING, loading: false });
  }

  const logout = () => dispatch({ type: AuthAction.LOGOUT });
  const unspoof = () => dispatch({ type: AuthAction.UNSPOOF });

  const credFail = (handler: () => void) => () => (handler(), [undefined, undefined]);

  async function fetchCredentials(token: string): Promise<[user: CDatabaseContract["User"], rp: Policy[]]> {
    const extraHeaders = getAuthHeader(token);
    const projectId = !!search.get("projectId") ? search.get("projectId") : project;
    const apiPath = `/${projectId}/${credentialApiPath}`;
    const credentials = await apiRequest({ subpath: apiPath, jsonify: true, extraHeaders });
    if (!credentials) throw Error("Could not authenticate!");
    const { user, policies } = credentials as CredentialDTO;
    if (!user || !policies) throw Error("Could not authenticate!");
    const rp = Policy.parseResourcePolicies<Resource>(policies);
    return [user as CDatabaseContract["User"], rp as Policy[]];
  }

  async function authInit() {
    const userToken = getUserToken();
    const spoofToken = getSpoofToken();
    const [[user, userPolicies], [spoofUser, spoofPolicies]] = await Promise.all([
      !!userToken ? fetchCredentials(userToken).catch(credFail(logout)) : [],
      !!spoofToken ? fetchCredentials(spoofToken).catch(credFail(unspoof)) : [],
    ]);

    if (!!user && !!userToken) await login(user, userToken, userPolicies);
    if (!!spoofUser && !!spoofToken) await spoof(spoofUser, spoofToken, spoofPolicies);
    dispatch({ type: AuthAction.INTIALIZE, initialized: true });
  }

  const context: AuthContextType = {
    ...state,
    logout,
    login,
    spoof,
    unspoof,
    authInit,
  };

  const contextValue = useMemo(() => context, [state, dispatch]);

  return <AuthContext.Provider value={contextValue}>{children}</AuthContext.Provider>;
};

export const getUserToken = () => window.localStorage.getItem(AuthStorageKeys.USER_TOKEN) ?? undefined;
export const getSpoofToken = () => window.localStorage.getItem(AuthStorageKeys.SPOOF_TOKEN) ?? undefined;
export const getActiveUserToken = () => getSpoofToken() ?? getUserToken() ?? undefined;
export const getAuthHeader = (token?: string) => (!!token ? { Authorization: `Bearer ${token}` } : undefined);
